package io.adjoe.sdk.cordova;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import android.content.Intent;
import android.util.Log;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CordovaWebView;
import org.apache.cordova.LOG;
import org.apache.cordova.PluginResult;
import org.apache.cordova.PluginResult.Status;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import io.adjoe.sdk.Adjoe;
import io.adjoe.sdk.AdjoeException;
import io.adjoe.sdk.AdjoeGender;
import io.adjoe.sdk.AdjoeInitialisationListener;
import io.adjoe.sdk.AdjoeNotInitializedException;
import io.adjoe.sdk.AdjoePayoutError;
import io.adjoe.sdk.AdjoePayoutListener;
import io.adjoe.sdk.AdjoeRewardListener;
import io.adjoe.sdk.AdjoeRewardResponse;
import io.adjoe.sdk.AdjoeRewardResponseError;

public class AdjoePlugin extends CordovaPlugin {

    private static final String LOGTAG = "[AdjoePlugin]";

    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
        Log.i(LOGTAG, "execute " + action + " with args " + args);
        PluginResult result = null;
        if (action == null) {
            return false;
        }

        if (action.equals("init")) {
            Log.i(LOGTAG, "adjoe SDK initialize");
            String sdkHash = args.getString(0);
            JSONObject options = args.getJSONObject(1);
            String subId1 = args.optString(2, null);
            String subId2 = args.optString(3, null);

            if (sdkHash.isEmpty()) {
                callbackContext.error("called init without sdkHash");
                return false;
            }

            executeInitialize(callbackContext, sdkHash, options, subId1, subId2);

        } else if (action.equals("showOfferwall")) {
            Log.i(LOGTAG, "adjoe SDK show offerwall");
            executeShowOfferwall(callbackContext, args.optString(0, null), args.optString(1, null));

        } else if (action.equals("canShowOfferwall")) {
            Log.i(LOGTAG, "adjoe SDK can show offerwall");
            result = executeCanShowOfferwall();

        } else if (action.equals("getRewards")) {
            Log.i(LOGTAG, "adjoe SDK rewards");
            executeGetRewards(callbackContext, args.optString(0, null), args.optString(1, null));

        } else if (action.equals("doPayout")) {
            Log.i(LOGTAG, "adjoe SDK payout");
            executeDoPayout(callbackContext, args.optString(0, null), args.optString(1, null));

        } else if (action.equals("setProfile")) {
            Log.i(LOGTAG, "adjoe SDK set profile");
            executeSetProfile(callbackContext, args.getString(0), args.getString(1), args.getString(2), args.optString(3, null), args.optString(4, null));

        } else if (action.equals("sendUserEvent")) {
            Log.i(LOGTAG, "adjoe SDK sendUserEvent");
            executeSendUserEvent(callbackContext, args.getInt(0), args.optString(1, null), args.optString(2, null));

        } else if (action.equals("getVersion")) {
            Log.i(LOGTAG, "adjoe SDK get version");
            result = executeGetVersion();

        } else if (action.equals("getVersionName")) {
            Log.i(LOGTAG, "adjoe SDK get version name");
            result = executeGetVersionName();

        } else if (action.equals("isInitialized")) {
            Log.i(LOGTAG, "adjoe SDK is initialized");
            result = executeIsInitialized();

        } else if (action.equals("hasAcceptedTOS")) {
            Log.i(LOGTAG, "adjoe SDK has accepted TOS");
            result = executeHasAcceptedTOS();

        } else if (action.equals("hasAcceptedUsagePermission")) {
            Log.i(LOGTAG, "adjoe SDK has accepted usage permission");
            result = executeHasAcceptedUsagePermission();

        } else if (action.equals("canUseOfferwallFeatures")) {
            Log.i(LOGTAG, "adjoe SDK can use offerwall features");
            result = executeCanUseOfferwallFeatures();

        } else if (action.equals("getUserId")) {
            Log.i(LOGTAG, "adjoe SDK get user Id");
            result = executeGetUserId();

        } else if (action.equals("a")) {
            Log.i(LOGTAG, "adjoe a");
            a(callbackContext, args.getBoolean(0));
        }

        if (result != null) {
            callbackContext.sendPluginResult(result);
        }
        return true;
    }

    private void executeInitialize(final CallbackContext callbackContext, final String sdkHash, final JSONObject options, final String subId1, final String subId2) {
        cordova.getThreadPool().execute(new Runnable() {
            public void run() {
                Adjoe.Options adjoeOptions = new Adjoe.Options();
                if (options != null) {
                    adjoeOptions.setUserId(options.optString("user_id", null));
                }
                Adjoe.init(cordova.getActivity(), sdkHash, adjoeOptions, subId1, subId2, new AdjoeInitialisationListener() {

                    @Override
                    public void onInitialisationFinished() {
                        Log.i(LOGTAG, "adjoe SDK initialized");
                        callbackContext.success();
                    }

                    @Override
                    public void onInitialisationError(Exception e) {
                        Log.e(LOGTAG, "adjoe SDK not initialized");
                        if (e != null) {
                            e.printStackTrace();
                            callbackContext.error(e.getMessage());
                        } else {
                            callbackContext.error("adjoe SDK could not be initialized");
                        }
                    }
                });
            }
        });
    }

    private void executeShowOfferwall(final CallbackContext callbackContext, final String subId1, final String subId2) {
        this.cordova.getActivity().runOnUiThread(new Runnable() {

            @Override
            public void run() {
                try {
                    Intent adjoeOfferwallIntent = Adjoe.getOfferwallIntent(cordova.getActivity(), subId1, subId2);
                    cordova.getActivity().startActivity(adjoeOfferwallIntent);

                    callbackContext.success();

                } catch (AdjoeNotInitializedException notInitializedException) {
                    Log.e(LOGTAG, "adjoe SDK not initialized", notInitializedException);
                    callbackContext.error(notInitializedException.getMessage());

                } catch (AdjoeException exception) {
                    Log.e(LOGTAG, "adjoe SDK threw an exception", exception);
                    callbackContext.error(exception.getMessage());
                }
            }
        });
    }

    private PluginResult executeCanShowOfferwall() throws JSONException {
        JSONObject message = new JSONObject();
        message.put("can_show", Adjoe.canShowOfferwall(cordova.getActivity()));
        return new PluginResult(PluginResult.Status.OK, message);
    }

    private void executeGetRewards(final CallbackContext callbackContext, final String subId1, final String subId2) {
        cordova.getThreadPool().execute(new Runnable() {
            public void run() {
                try {
                    Adjoe.requestRewards(cordova.getActivity(), subId1, subId2, new AdjoeRewardListener() {

                        @Override
                        public void onUserReceivesReward(AdjoeRewardResponse adjoeRewardResponse) {
                            // successfully requested the rewards
                            // get the total amount of coins which the user has collected
                            int reward = adjoeRewardResponse.getReward();
                            // get the amount of coins which are available for payout
                            int availableForPayout = adjoeRewardResponse.getAvailablePayoutCoins();
                            // get the amount of coins which the user has already spent
                            int alreadySpentCoins = adjoeRewardResponse.getAlreadySpentCoins();


                            JSONObject json = new JSONObject();
                            try {
                                json.put("reward", reward);
                                json.put("available_for_payout", availableForPayout);
                                json.put("already_spent_coins", alreadySpentCoins);
                            }
                            catch (JSONException e){}

                            callbackContext.sendPluginResult(new PluginResult(PluginResult.Status.OK, json));
                        }

                        // todo: corvin, when executed.
                        @Override
                        public void onUserReceivesRewardError(AdjoeRewardResponseError adjoeRewardResponseError) {
                            // an error occurred while requesting the rewards
                            // you can try to get additional information about the error for debugging
                            Log.e(LOGTAG, "adjoe  onUserReceivesRewardError" + adjoeRewardResponseError.toString());

                            if (adjoeRewardResponseError.getException() != null) {
                                adjoeRewardResponseError.getException().printStackTrace();
                                callbackContext.error(adjoeRewardResponseError.getException().getMessage());
                            } else {
                                callbackContext.error("adjoe SDK could not fetch rewards");
                            }
                        }
                    });
                } catch (AdjoeNotInitializedException e) {
                    Log.e(LOGTAG, "adjoe SDK not initialized", e);
                    callbackContext.error(e.getMessage());
                }
            }
        });
    }

    private void executeDoPayout(final CallbackContext callbackContext, final String subId1, final String subId2) {
        cordova.getThreadPool().execute(new Runnable() {
            public void run() {

                try {
                    Adjoe.doPayout(cordova.getActivity(), subId1, subId2, new AdjoePayoutListener() {

                        @Override
                        public void onPayoutExecuted(int coins) {
                            // successfully paid out 'coins' coins
                            JSONObject json = new JSONObject();
                            try {
                                json.put("payout_coins", coins);
                            }
                            catch (JSONException e){}

                            callbackContext.sendPluginResult(new PluginResult(PluginResult.Status.OK, json));
                        }

                        @Override
                        public void onPayoutError(AdjoePayoutError adjoePayoutError) {

                            // an error occurred while paying out the coins

                            // get information about why it failed
                            // 'reason' is one of
                            // - AdjoePayoutError.UNKNOWN
                            // - AdjoePayoutError.NOT_ENOUGH_COINS
                            // - AdjoePayoutError.TOS_NOT_ACCEPTED
                            int reason = adjoePayoutError.getReason();

                            JSONObject message = new JSONObject();
                            try {
                                message.put("reason", reason);
                            } catch (JSONException e) {}

                            // if available, get more information about the error

                            if (adjoePayoutError.getException() != null) {
                                adjoePayoutError.getException().printStackTrace();
                                try {
                                    message.put("exception", adjoePayoutError.getException().getMessage());
                                } catch (JSONException e) {}
                            }
                            callbackContext.sendPluginResult(new PluginResult(PluginResult.Status.ERROR, message));
                        }
                    });
                } catch (AdjoeNotInitializedException e) {
                    // make sure to initialize the adjoe SDK first
                    Log.e(LOGTAG, "adjoe SDK not initialized", e);
                    callbackContext.error(e.getMessage());
                }
            }
        });
    }

    private void executeSetProfile(final CallbackContext callbackContext, final String source, final String genderString, final String birthdayString, final String subId1, final String subId2) {
        cordova.getThreadPool().execute(new Runnable() {

            @Override
            public void run() {
                AdjoeGender gender;
                if (genderString.equalsIgnoreCase("male")) {
                    gender = AdjoeGender.MALE;
                } else if (genderString.equalsIgnoreCase("female")) {
                    gender = AdjoeGender.FEMALE;
                } else {
                    gender = AdjoeGender.UNKNOWN;
                }

                Date birthday = null;
                try {
                    birthday = new SimpleDateFormat("yyyy-MM-dd").parse(birthdayString);
                } catch (ParseException e) {
                    Log.e(LOGTAG, "adjoe SDK birthday is in wrong format: " + birthdayString + "; expected yyyy-MM-dd", e);
                    callbackContext.error(e.getMessage());
                    return;
                }

                try {
                    Adjoe.setProfile(cordova.getActivity(), source, gender, birthday, subId1, subId2);

                    callbackContext.success();
                } catch(AdjoeNotInitializedException notInitializedException) {
                    Log.e(LOGTAG, "adjoe SDK not initialized", notInitializedException);
                    callbackContext.error(notInitializedException.getMessage());
                }
            }
        });
    }

    private void a(final CallbackContext callbackContext, final boolean a) {
        cordova.getThreadPool().execute(new Runnable() {

            @Override
            public void run() {
                    Adjoe.a(cordova.getActivity(), a);
                    callbackContext.success();
            }
        });
    }

    private void executeSendUserEvent(final CallbackContext callbackContext, final int eventId, final String subId1, final String subId2) {
        cordova.getThreadPool().execute(new Runnable() {

            @Override
            public void run() {
                try {
                    Adjoe.sendUserEvent(cordova.getActivity(), eventId, null, subId1, subId2);

                    callbackContext.success();
                } catch (AdjoeNotInitializedException e) {
                    Log.e(LOGTAG, "adjoe SDK not initialized", e);
                    callbackContext.error(e.getMessage());
                }
            }
        });
    }

    private PluginResult executeGetVersion() throws JSONException {
        JSONObject message = new JSONObject();
        message.put("version", Adjoe.getVersion());
        return new PluginResult(PluginResult.Status.OK, message);
    }

    private PluginResult executeGetVersionName() throws JSONException {
        JSONObject message = new JSONObject();
        message.put("version_name", Adjoe.getVersionName());
        return new PluginResult(PluginResult.Status.OK, message);
    }

    private PluginResult executeIsInitialized() throws JSONException {
        JSONObject message = new JSONObject();
        message.put("initialized", Adjoe.isInitialized());
        return new PluginResult(PluginResult.Status.OK, message);
    }

    private PluginResult executeHasAcceptedTOS() throws JSONException {
        JSONObject message = new JSONObject();
        message.put("accepted", Adjoe.hasAcceptedTOS(cordova.getActivity()));
        return new PluginResult(PluginResult.Status.OK, message);
    }

    private PluginResult executeHasAcceptedUsagePermission() throws JSONException {
        JSONObject message = new JSONObject();
        message.put("accepted", Adjoe.hasAcceptedUsagePermission(cordova.getActivity()));
        return new PluginResult(PluginResult.Status.OK, message);
    }

    private PluginResult executeCanUseOfferwallFeatures() throws JSONException {
        JSONObject message = new JSONObject();
        message.put("can_use", Adjoe.canUseOfferwallFeatures(cordova.getActivity()));
        return new PluginResult(PluginResult.Status.OK, message);
    }

    private PluginResult executeGetUserId() throws JSONException {
        JSONObject message = new JSONObject();
        message.put("user_id", Adjoe.getUserId(cordova.getActivity()));
        return new PluginResult(PluginResult.Status.OK, message);
    }

}
